//=============================================================================
// SII BrainPad(TM) TiPO Communication Utility
// Copyright(c)1999 Yamamoto Takanori.
//=============================================================================
#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <setjmp.h>

#include "tipo.h"
#include "tipoware.h"
#include "exitstatus.h"

//G.Variables
jmp_buf env;

int tipoware(const char *sDev, int nBps, const char *sCmd, const char *sArg);
void tipowaretimeout(void);
void printusage(const char *sCommand);
int chkcmd(const char *s, const char *t);

int chkcmd(const char *s, const char *t){
  int ns,nt;
  ns=strlen(s)-1; nt=strlen(t)-1;

  if(ns<0||nt<0||ns<nt){return -1;}
  while(nt>=0){
    if(s[ns]!=t[nt]){return -1;}
    ns--; nt--;
  }
  return 0;
}

void tipowaretimeout(void){
  fprintf(stderr, "serial timeout occurred.\n");
  longjmp(env, 0);
}

int tipoware(const char *sDev, int nBps, const char *sCmd, const char *sArg){
  if(!chkcmd(sCmd, "twstat")){
    //TWSTAT
    static int nCnt; nCnt=0;
    TiPO tipo;
    if(tipo.open(sDev, nBps)){return EXIT_DEVICEOPENERR;}
    tipo.setcallbackfunc_timeout(tipowaretimeout);
    setjmp(env);
    if(nCnt++>=3){tipo.close(); return EXIT_TIMEOUT;}
    printf("%ld\n", tipo.twstat());
    tipo.close();
    return EXIT_NORMAL;
  }else if(!chkcmd(sCmd, "twexit")){
    //TWEXIT
    static int nCnt; nCnt=0;
    TiPO tipo;
    if(tipo.open(sDev, nBps)){return EXIT_DEVICEOPENERR;}
    tipo.setcallbackfunc_timeout(tipowaretimeout);
    setjmp(env);
    if(nCnt++>=3){tipo.close(); return EXIT_TIMEOUT;}
    tipo.twexit();
    tipo.close();
    return EXIT_NORMAL;
  }else if(!chkcmd(sCmd, "verup")){
    //VERUP(TWSTAT,TWVERS,TWVERU)
    static int nCnt; nCnt=0;
    TiPO tipo;
    if(tipo.open(sDev, nBps)){return EXIT_DEVICEOPENERR;}
    tipo.setcallbackfunc_timeout(tipowaretimeout);
    setjmp(env);
    if(nCnt++>=3){tipo.close(); return EXIT_TIMEOUT;}
    if(tipo.twstat()<8129536){
      fprintf(stderr, "TiPO DISKSPACE IS NOT ENOUGH\n");
      tipo.close();
      return EXIT_DISKNOTENOUGH;
    }
    if(tipo.twvers()){
      fprintf(stderr, "BATTERY LIFE IS NOT ENOUGH\n");
      tipo.close();
      return EXIT_BATTNOTENOUGH;
    }
    int ret;
    if((ret=tipo.twveru("os.bz"))){tipo.close(); return ret;}
    if((ret=tipo.twveru("ap.bz"))){tipo.close(); return ret;}
    tipo.twexit();
    tipo.close();
    return EXIT_NORMAL;
  }else if(!chkcmd(sCmd, "twvers")){
    //TWVERS
    static int nCnt; nCnt=0;
    TiPO tipo;
    if(tipo.open(sDev, nBps)){return EXIT_DEVICEOPENERR;}
    tipo.setcallbackfunc_timeout(tipowaretimeout);
    setjmp(env);
    if(nCnt++>=3){tipo.close(); return EXIT_TIMEOUT;}
    tipo.twvers();
    tipo.close();
    return EXIT_NORMAL;
  }else if(!chkcmd(sCmd, "twlist")){
    //TWLIST
    static int nCnt; nCnt=0;
    TiPO tipo;
    if(tipo.open(sDev, nBps)){return EXIT_DEVICEOPENERR;}
    tipo.setcallbackfunc_timeout(tipowaretimeout);
    setjmp(env);
    if(nCnt++>=3){tipo.close(); return EXIT_TIMEOUT;}
    ezlist heno;
    tipo.twlist(&heno);
    ezatom *ptr;
    heno.gofirst(&ptr);
    while(!heno.forward(&ptr)){
      char *str=heno.getdata(&ptr);
      printf("%s\n", str);
    }
    tipo.close();
    return EXIT_NORMAL;
  }else if(!chkcmd(sCmd, "twdele")){
    //TWDELE
    if(sArg==NULL){printusage(sCmd); return EXIT_SHOWUSAGE;}
    static int nCnt; nCnt=0;
    TiPO tipo;
    if(tipo.open(sDev, nBps)){return EXIT_DEVICEOPENERR;}
    tipo.setcallbackfunc_timeout(tipowaretimeout);
    setjmp(env);
    if(nCnt++>=3){tipo.close(); return EXIT_TIMEOUT;}
    int ret;
    if((ret=tipo.twdele(sArg))){tipo.close(); return ret;}
    tipo.close();
    return EXIT_NORMAL;
  }else if(!chkcmd(sCmd, "twsend")){
    //TWSEND
    if(sArg==NULL){printusage(sCmd); return EXIT_SHOWUSAGE;}
    static int nCnt; nCnt=0;
    TiPO tipo;
    if(tipo.open(sDev, nBps)){return EXIT_DEVICEOPENERR;}
    tipo.setcallbackfunc_timeout(tipowaretimeout);
    setjmp(env);
    if(nCnt++>=3){tipo.close(); return EXIT_TIMEOUT;}
    int ret;
    if((ret=tipo.twsend(sArg))){tipo.close(); return ret;}
    tipo.close();
    return EXIT_NORMAL;
  }else if(!chkcmd(sCmd, "twrecv")){
    //TWRECV
    if(sArg==NULL){printusage(sCmd); return EXIT_SHOWUSAGE;}
    static int nCnt; nCnt=0;
    TiPO tipo;
    if(tipo.open(sDev, nBps)){return EXIT_DEVICEOPENERR;}
    tipo.setcallbackfunc_timeout(tipowaretimeout);
    setjmp(env);
    if(nCnt++>=3){tipo.close(); return EXIT_TIMEOUT;}
    int ret;
    if((ret=tipo.twrecv(sArg))){tipo.close(); return ret;}
    tipo.close();
    return EXIT_NORMAL;
  }
  //Unk. Command
  printusage("tipoware");
  return EXIT_SHOWUSAGE;
}

int main(int argc, char *argv[]){
  extern char *optarg;
  extern int optind;
  char *sDev = DEVICE;
  static char *sCmd = argv[0];
  int ch;

  while((ch = getopt(argc, argv, "c:C:d:D:?")) != -1){
    switch(ch){
      case 'd':
      case 'D':
        sDev = optarg;
        break;
      case 'c':
      case 'C':
        sCmd = optarg;
        break;
      case '?':
      case 'h':
      case 'H':
        printusage(sCmd);
        return EXIT_SHOWUSAGE;
        break;
    }
  }

  char *sArg = argv[optind];
  return tipoware(sDev, B19200, sCmd, sArg);
}

void printusage(const char *sCommand){
  fprintf(stderr, "This is TiPOWARE, version 0.12\n");
  fprintf(stderr, "Copyright(c)1999 Yamamoto Takanori<tatyana@miko.org>\n\n");
  fprintf(stderr, "usage: %s [-[cC]command] [-[dD]device] [file]\n", sCommand);
}
